/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                 Java Language Spatial FOG SDK, Version 6.0                 */
/*                    Copyright 2022, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2022 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* Installation Alignment */
public class ANPacket185
{
	public static final int LENGTH = 73;

	public Boolean permanent;
	public float[][] alignmentDCM;
	/* metres */
	public float[] gnssAntennaOffset;
	public float[] odometerOffset;
	public float[] externalDataOffset;
	
	public ANPacket185()
	{
		alignmentDCM = new float[3][3];
		gnssAntennaOffset = new float[3];
		odometerOffset = new float[3];
		externalDataOffset = new float[3];
	}

	public ANPacket185(ANPacket packet)
	{
		permanent = packet.data[0] != 0;
		alignmentDCM = new float[3][3];
		alignmentDCM[0][0] = TypeConversion.bytesToFloat(packet.data, 1);
		alignmentDCM[0][1] = TypeConversion.bytesToFloat(packet.data, 5);
		alignmentDCM[0][2] = TypeConversion.bytesToFloat(packet.data, 9);
		alignmentDCM[1][0] = TypeConversion.bytesToFloat(packet.data, 13);
		alignmentDCM[1][1] = TypeConversion.bytesToFloat(packet.data, 17);
		alignmentDCM[1][2] = TypeConversion.bytesToFloat(packet.data, 21);
		alignmentDCM[2][0] = TypeConversion.bytesToFloat(packet.data, 25);
		alignmentDCM[2][1] = TypeConversion.bytesToFloat(packet.data, 29);
		alignmentDCM[2][2] = TypeConversion.bytesToFloat(packet.data, 33);
		gnssAntennaOffset = new float[3];
		gnssAntennaOffset[0] = TypeConversion.bytesToFloat(packet.data, 37);
		gnssAntennaOffset[1] = TypeConversion.bytesToFloat(packet.data, 41);
		gnssAntennaOffset[2] = TypeConversion.bytesToFloat(packet.data, 45);
		odometerOffset = new float[3];
		odometerOffset[0] = TypeConversion.bytesToFloat(packet.data, 49);
		odometerOffset[1] = TypeConversion.bytesToFloat(packet.data, 53);
		odometerOffset[2] = TypeConversion.bytesToFloat(packet.data, 57);
		externalDataOffset = new float[3];
		externalDataOffset[0] = TypeConversion.bytesToFloat(packet.data, 61);
		externalDataOffset[1] = TypeConversion.bytesToFloat(packet.data, 65);
		externalDataOffset[2] = TypeConversion.bytesToFloat(packet.data, 69);
	}
	
	public ANPacket toANPacket()
	{	
		ANPacket packet = new ANPacket(73, ANPacket.PACKET_ID_INSTALLATION_ALIGNMENT);
		packet.data[0] = (byte) (permanent ? 1 : 0);
		TypeConversion.floatToBytes(packet.data, 1, alignmentDCM[0][0]);
		TypeConversion.floatToBytes(packet.data, 5, alignmentDCM[0][1]);
		TypeConversion.floatToBytes(packet.data, 9, alignmentDCM[0][2]);
		TypeConversion.floatToBytes(packet.data, 13, alignmentDCM[1][0]);
		TypeConversion.floatToBytes(packet.data, 17, alignmentDCM[1][1]);
		TypeConversion.floatToBytes(packet.data, 21, alignmentDCM[1][2]);
		TypeConversion.floatToBytes(packet.data, 25, alignmentDCM[2][0]);
		TypeConversion.floatToBytes(packet.data, 29, alignmentDCM[2][1]);
		TypeConversion.floatToBytes(packet.data, 33, alignmentDCM[2][2]);
		TypeConversion.floatToBytes(packet.data, 37, gnssAntennaOffset[0]);
		TypeConversion.floatToBytes(packet.data, 41, gnssAntennaOffset[1]);
		TypeConversion.floatToBytes(packet.data, 45, gnssAntennaOffset[2]);
		TypeConversion.floatToBytes(packet.data, 49, odometerOffset[0]);
		TypeConversion.floatToBytes(packet.data, 53, odometerOffset[1]);
		TypeConversion.floatToBytes(packet.data, 57, odometerOffset[2]);
		TypeConversion.floatToBytes(packet.data, 61, externalDataOffset[0]);
		TypeConversion.floatToBytes(packet.data, 65, externalDataOffset[1]);
		TypeConversion.floatToBytes(packet.data, 69, externalDataOffset[2]);
		return packet;
	}
	
	public double[] dcmToEuler()
	{
		double[] euler = new double[3];
		euler[0] = Math.atan2(alignmentDCM[1][2], alignmentDCM[2][2]);
		euler[1] = -Math.asin(alignmentDCM[0][2]);
		euler[2] = Math.atan2(alignmentDCM[0][1], alignmentDCM[0][0]);
		return euler;
	}
	
	public void eulerToDCM(double[] euler)
	{
		double sin_phi = Math.sin(euler[0]);
		double sin_theta = Math.sin(euler[1]);
		double sin_psi = Math.sin(euler[2]);

		double cos_phi = Math.cos(euler[0]);
		double cos_theta = Math.cos(euler[1]);
		double cos_psi = Math.cos(euler[2]);

		alignmentDCM[0][0] = (float) (cos_psi * cos_theta);
		alignmentDCM[0][1] = (float) (sin_psi * cos_theta);
		alignmentDCM[0][2] = (float) -sin_theta;

		alignmentDCM[1][0] = (float) (-sin_psi * cos_phi + cos_psi * sin_theta * sin_phi);
		alignmentDCM[1][1] = (float) (cos_psi * cos_phi + sin_psi * sin_theta * sin_phi);
		alignmentDCM[1][2] = (float) (cos_theta * sin_phi);

		alignmentDCM[2][0] = (float) (sin_psi * sin_phi + cos_psi * sin_theta * cos_phi);
		alignmentDCM[2][1] = (float) (-cos_psi * sin_phi + sin_psi * sin_theta * cos_phi);
		alignmentDCM[2][2] = (float) (cos_theta * cos_phi);
	}
}
